package cn.newfeifan.mall.sale.service.merchant;

import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.module.system.mq.message.login.LoginSendMessage;
import cn.newfeifan.mall.sale.controller.admin.merchant.vo.MerchantPageReqVO;
import cn.newfeifan.mall.sale.controller.admin.merchant.vo.MerchantRespVO;
import cn.newfeifan.mall.sale.controller.admin.merchant.vo.MerchantSaveReqVO;
import cn.newfeifan.mall.sale.dal.dataobject.merchant.MerchantDO;

import javax.validation.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 商户 Service 接口
 *
 * @author 非繁人
 */
public interface MerchantService {

    /**
     * 创建商户
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    HashMap<String, Long> createMerchant(@Valid MerchantSaveReqVO createReqVO, String mobile);

    /**
     * 更新商户
     *
     * @param updateReqVO 更新信息
     */
    void updateMerchant(@Valid MerchantSaveReqVO updateReqVO);

    /**
     * 删除商户
     *
     * @param id 编号
     */
    void deleteMerchant(Long id);

    /**
     * 获得商户
     *
     * @param id 编号
     * @return 商户
     */
    MerchantDO getMerchant(Long id);

    /**
     * 获得商户分页
     *
     * @param pageReqVO 分页查询
     * @return 商户分页
     */
    PageResult<MerchantDO> getMerchantPage(MerchantPageReqVO pageReqVO);

    List<MerchantRespVO> salesVolume(Map<Long, MerchantRespVO> map);
    void doSendMail(LoginSendMessage message);

    /**
     * 根据商户申请id查询商户
     * @param id 商户申请id
     * @return 商户
     */
    MerchantDO selectByApplyId(Long id);

    /**
     * 更新商户
     * @param merchantDO 商户
     */
    void updateMerchant(MerchantDO merchantDO);
}